@@ -1,4 +1,4 @@
-/* Copyright (C) 2001, 2009, 2010 Free Software Foundation, Inc.
+/* Copyright (C) 2001, 2009, 2010, 2011 Free Software Foundation, Inc.
  * 
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -23,8 +23,10 @@
 #include <string.h>
 
 #include "_scm.h"
+#include "threads.h"
 #include "instructions.h"
 
+
 struct scm_instruction {
   enum scm_opcode opcode;	/* opcode */
   const char *name;		/* instruction name */
@@ -45,11 +47,15 @@ struct scm_instruction {
   } while (0)
 
 
+static scm_i_pthread_mutex_t itable_lock = SCM_I_PTHREAD_MUTEX_INITIALIZER;
+
+
 static struct scm_instruction*
 fetch_instruction_table ()
 {
   static struct scm_instruction *table = NULL;
 
+  scm_i_pthread_mutex_lock (&itable_lock);
   if (SCM_UNLIKELY (!table))
     {
       size_t bytes = SCM_VM_NUM_INSTRUCTIONS * sizeof(struct scm_instruction);
@@ -71,28 +77,36 @@ fetch_instruction_table ()
             table[i].symname = SCM_BOOL_F;
         }
     }
+  scm_i_pthread_mutex_unlock (&itable_lock);
+
   return table;
 }
 
+static SCM instructions_by_name;
+
+static void
+init_instructions_by_name (void)
+{
+  struct scm_instruction *table = fetch_instruction_table ();
+  unsigned int i;
+
+  instructions_by_name =
+    scm_make_hash_table (SCM_I_MAKINUM (SCM_VM_NUM_INSTRUCTIONS));
+
+  for (i = 0; i < SCM_VM_NUM_INSTRUCTIONS; i++)
+    if (scm_is_true (table[i].symname))
+      scm_hashq_set_x (instructions_by_name, table[i].symname,
+                       SCM_I_MAKINUM (i));
+}
+
 static struct scm_instruction *
 scm_lookup_instruction_by_name (SCM name)
 {
-  static SCM instructions_by_name = SCM_BOOL_F;
+  static scm_i_pthread_once_t once = SCM_I_PTHREAD_ONCE_INIT;
   struct scm_instruction *table = fetch_instruction_table ();
   SCM op;
 
-  if (SCM_UNLIKELY (scm_is_false (instructions_by_name)))
-    {
-      unsigned int i;
-
-      instructions_by_name =
-        scm_make_hash_table (SCM_I_MAKINUM (SCM_VM_NUM_INSTRUCTIONS));
-
-      for (i = 0; i < SCM_VM_NUM_INSTRUCTIONS; i++)
-        if (scm_is_true (table[i].symname))
-          scm_hashq_set_x (instructions_by_name, table[i].symname,
-                           SCM_I_MAKINUM (i));
-    }
+  scm_i_pthread_once (&once, init_instructions_by_name);
 
   op = scm_hashq_ref (instructions_by_name, name, SCM_UNDEFINED);
   if (SCM_I_INUMP (op))
